.vColorField {
  display: flex;
  width: 200px;
  justify-content: space-between;
}

.vColorField .pickr .pcr-button {
  width: 4em;
}

.pcr-app[data-theme=monolith] {
  width: 16.4em;
  font-size: 16px;
}

.pcr-app .pcr-interaction .pcr-result {
  flex: 1 1 10em;
  min-width: 10em;
}

.pcr-app .pcr-interaction .pcr-save {
  margin-left: auto;
  padding: .45em .75em;
  font-size: 0.875em;
  background: #28a745;
  order: 1;
}

.pcr-app .pcr-interaction .pcr-clear {
  padding: .45em .75em;
  font-size: 0.875em;
}
